/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.1 (build: 2013-08-01 15:32:38 UTC)
 *  on 2013-08-06 at 21:07:17 UTC 
 */

package com.google.api.services.reseller;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Reseller (v1).
 *
 * <p>
 * Lets you create and manage your customers and their subscriptions.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/google-apps/reseller/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link ResellerRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Reseller extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Enterprise Apps Reseller API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "apps/reseller/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Reseller(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Reseller(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Customers collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Reseller reseller = new Reseller(...);}
   *   {@code Reseller.Customers.List request = reseller.customers().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Customers customers() {
    return new Customers();
  }

  /**
   * The "customers" collection of methods.
   */
  public class Customers {

    /**
     * Gets a customer resource if one exists and is owned by the reseller.
     *
     * Create a request for the method "customers.get".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @return the request
     */
    public Get get(String customerId) throws java.io.IOException {
      Get result = new Get(customerId);
      initialize(result);
      return result;
    }

    public class Get extends ResellerRequest<com.google.api.services.reseller.model.Customer> {

      private static final String REST_PATH = "customers/{customerId}";

      /**
       * Gets a customer resource if one exists and is owned by the reseller.
       *
       * Create a request for the method "customers.get".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @since 1.13
       */
      protected Get(String customerId) {
        super(Reseller.this, "GET", REST_PATH, null, com.google.api.services.reseller.model.Customer.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Get setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

    }
    /**
     * Creates a customer resource if one does not already exist.
     *
     * Create a request for the method "customers.insert".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.reseller.model.Customer}
     * @return the request
     */
    public Insert insert(com.google.api.services.reseller.model.Customer content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends ResellerRequest<com.google.api.services.reseller.model.Customer> {

      private static final String REST_PATH = "customers";

      /**
       * Creates a customer resource if one does not already exist.
       *
       * Create a request for the method "customers.insert".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       * <p> {@link Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.reseller.model.Customer}
       * @since 1.13
       */
      protected Insert(com.google.api.services.reseller.model.Customer content) {
        super(Reseller.this, "POST", REST_PATH, content, com.google.api.services.reseller.model.Customer.class);
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /**
     * An auth token needed for inserting a customer for which domain already exists. Can be
     * generated at https://www.google.com/a/cpanel//TransferToken. Optional.
     */
      @com.google.api.client.util.Key
      private String customerAuthToken;

      /** An auth token needed for inserting a customer for which domain already exists. Can be generated at
     https://www.google.com/a/cpanel//TransferToken. Optional.
       */
      public String getCustomerAuthToken() {
        return customerAuthToken;
      }

      /**
     * An auth token needed for inserting a customer for which domain already exists. Can be
     * generated at https://www.google.com/a/cpanel//TransferToken. Optional.
     */
      public Insert setCustomerAuthToken(String customerAuthToken) {
        this.customerAuthToken = customerAuthToken;
        return this;
      }

    }
    /**
     * Update a customer resource if one it exists and is owned by the reseller. This method supports
     * patch semantics.
     *
     * Create a request for the method "customers.patch".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param content the {@link com.google.api.services.reseller.model.Customer}
     * @return the request
     */
    public Patch patch(String customerId, com.google.api.services.reseller.model.Customer content) throws java.io.IOException {
      Patch result = new Patch(customerId, content);
      initialize(result);
      return result;
    }

    public class Patch extends ResellerRequest<com.google.api.services.reseller.model.Customer> {

      private static final String REST_PATH = "customers/{customerId}";

      /**
       * Update a customer resource if one it exists and is owned by the reseller. This method supports
       * patch semantics.
       *
       * Create a request for the method "customers.patch".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Patch#execute()} method to invoke the remote operation.
       * <p> {@link Patch#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param content the {@link com.google.api.services.reseller.model.Customer}
       * @since 1.13
       */
      protected Patch(String customerId, com.google.api.services.reseller.model.Customer content) {
        super(Reseller.this, "PATCH", REST_PATH, content, com.google.api.services.reseller.model.Customer.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public Patch setAlt(String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Patch setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

    }
    /**
     * Update a customer resource if one it exists and is owned by the reseller.
     *
     * Create a request for the method "customers.update".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param content the {@link com.google.api.services.reseller.model.Customer}
     * @return the request
     */
    public Update update(String customerId, com.google.api.services.reseller.model.Customer content) throws java.io.IOException {
      Update result = new Update(customerId, content);
      initialize(result);
      return result;
    }

    public class Update extends ResellerRequest<com.google.api.services.reseller.model.Customer> {

      private static final String REST_PATH = "customers/{customerId}";

      /**
       * Update a customer resource if one it exists and is owned by the reseller.
       *
       * Create a request for the method "customers.update".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Update#execute()} method to invoke the remote operation.
       * <p> {@link Update#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param content the {@link com.google.api.services.reseller.model.Customer}
       * @since 1.13
       */
      protected Update(String customerId, com.google.api.services.reseller.model.Customer content) {
        super(Reseller.this, "PUT", REST_PATH, content, com.google.api.services.reseller.model.Customer.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public Update setAlt(String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Update setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the Subscriptions collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Reseller reseller = new Reseller(...);}
   *   {@code Reseller.Subscriptions.List request = reseller.subscriptions().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Subscriptions subscriptions() {
    return new Subscriptions();
  }

  /**
   * The "subscriptions" collection of methods.
   */
  public class Subscriptions {

    /**
     * Changes the plan of a subscription
     *
     * Create a request for the method "subscriptions.changePlan".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link ChangePlan#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @param content the {@link com.google.api.services.reseller.model.ChangePlanRequest}
     * @return the request
     */
    public ChangePlan changePlan(String customerId, String subscriptionId, com.google.api.services.reseller.model.ChangePlanRequest content) throws java.io.IOException {
      ChangePlan result = new ChangePlan(customerId, subscriptionId, content);
      initialize(result);
      return result;
    }

    public class ChangePlan extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}/changePlan";

      /**
       * Changes the plan of a subscription
       *
       * Create a request for the method "subscriptions.changePlan".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link ChangePlan#execute()} method to invoke the remote
       * operation. <p> {@link ChangePlan#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @param content the {@link com.google.api.services.reseller.model.ChangePlanRequest}
       * @since 1.13
       */
      protected ChangePlan(String customerId, String subscriptionId, com.google.api.services.reseller.model.ChangePlanRequest content) {
        super(Reseller.this, "POST", REST_PATH, content, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
      }

      @Override
      public ChangePlan setAlt(String alt) {
        return (ChangePlan) super.setAlt(alt);
      }

      @Override
      public ChangePlan setFields(String fields) {
        return (ChangePlan) super.setFields(fields);
      }

      @Override
      public ChangePlan setKey(String key) {
        return (ChangePlan) super.setKey(key);
      }

      @Override
      public ChangePlan setOauthToken(String oauthToken) {
        return (ChangePlan) super.setOauthToken(oauthToken);
      }

      @Override
      public ChangePlan setPrettyPrint(Boolean prettyPrint) {
        return (ChangePlan) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ChangePlan setQuotaUser(String quotaUser) {
        return (ChangePlan) super.setQuotaUser(quotaUser);
      }

      @Override
      public ChangePlan setUserIp(String userIp) {
        return (ChangePlan) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public ChangePlan setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public ChangePlan setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

    }
    /**
     * Changes the renewal settings of a subscription
     *
     * Create a request for the method "subscriptions.changeRenewalSettings".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link ChangeRenewalSettings#execute()} method to invoke the remote
     * operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @param content the {@link com.google.api.services.reseller.model.RenewalSettings}
     * @return the request
     */
    public ChangeRenewalSettings changeRenewalSettings(String customerId, String subscriptionId, com.google.api.services.reseller.model.RenewalSettings content) throws java.io.IOException {
      ChangeRenewalSettings result = new ChangeRenewalSettings(customerId, subscriptionId, content);
      initialize(result);
      return result;
    }

    public class ChangeRenewalSettings extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}/changeRenewalSettings";

      /**
       * Changes the renewal settings of a subscription
       *
       * Create a request for the method "subscriptions.changeRenewalSettings".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link ChangeRenewalSettings#execute()} method to invoke the
       * remote operation. <p> {@link ChangeRenewalSettings#initialize(AbstractGoogleClientRequest)}
       * must be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @param content the {@link com.google.api.services.reseller.model.RenewalSettings}
       * @since 1.13
       */
      protected ChangeRenewalSettings(String customerId, String subscriptionId, com.google.api.services.reseller.model.RenewalSettings content) {
        super(Reseller.this, "POST", REST_PATH, content, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
      }

      @Override
      public ChangeRenewalSettings setAlt(String alt) {
        return (ChangeRenewalSettings) super.setAlt(alt);
      }

      @Override
      public ChangeRenewalSettings setFields(String fields) {
        return (ChangeRenewalSettings) super.setFields(fields);
      }

      @Override
      public ChangeRenewalSettings setKey(String key) {
        return (ChangeRenewalSettings) super.setKey(key);
      }

      @Override
      public ChangeRenewalSettings setOauthToken(String oauthToken) {
        return (ChangeRenewalSettings) super.setOauthToken(oauthToken);
      }

      @Override
      public ChangeRenewalSettings setPrettyPrint(Boolean prettyPrint) {
        return (ChangeRenewalSettings) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ChangeRenewalSettings setQuotaUser(String quotaUser) {
        return (ChangeRenewalSettings) super.setQuotaUser(quotaUser);
      }

      @Override
      public ChangeRenewalSettings setUserIp(String userIp) {
        return (ChangeRenewalSettings) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public ChangeRenewalSettings setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public ChangeRenewalSettings setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

    }
    /**
     * Changes the seats configuration of a subscription
     *
     * Create a request for the method "subscriptions.changeSeats".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link ChangeSeats#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @param content the {@link com.google.api.services.reseller.model.Seats}
     * @return the request
     */
    public ChangeSeats changeSeats(String customerId, String subscriptionId, com.google.api.services.reseller.model.Seats content) throws java.io.IOException {
      ChangeSeats result = new ChangeSeats(customerId, subscriptionId, content);
      initialize(result);
      return result;
    }

    public class ChangeSeats extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}/changeSeats";

      /**
       * Changes the seats configuration of a subscription
       *
       * Create a request for the method "subscriptions.changeSeats".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link ChangeSeats#execute()} method to invoke the remote
       * operation. <p> {@link ChangeSeats#initialize(AbstractGoogleClientRequest)} must be called to
       * initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @param content the {@link com.google.api.services.reseller.model.Seats}
       * @since 1.13
       */
      protected ChangeSeats(String customerId, String subscriptionId, com.google.api.services.reseller.model.Seats content) {
        super(Reseller.this, "POST", REST_PATH, content, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
      }

      @Override
      public ChangeSeats setAlt(String alt) {
        return (ChangeSeats) super.setAlt(alt);
      }

      @Override
      public ChangeSeats setFields(String fields) {
        return (ChangeSeats) super.setFields(fields);
      }

      @Override
      public ChangeSeats setKey(String key) {
        return (ChangeSeats) super.setKey(key);
      }

      @Override
      public ChangeSeats setOauthToken(String oauthToken) {
        return (ChangeSeats) super.setOauthToken(oauthToken);
      }

      @Override
      public ChangeSeats setPrettyPrint(Boolean prettyPrint) {
        return (ChangeSeats) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public ChangeSeats setQuotaUser(String quotaUser) {
        return (ChangeSeats) super.setQuotaUser(quotaUser);
      }

      @Override
      public ChangeSeats setUserIp(String userIp) {
        return (ChangeSeats) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public ChangeSeats setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public ChangeSeats setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

    }
    /**
     * Cancels/Downgrades a subscription.
     *
     * Create a request for the method "subscriptions.delete".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @param deletionType Whether the subscription is to be fully cancelled or downgraded
     * @return the request
     */
    public Delete delete(String customerId, String subscriptionId, String deletionType) throws java.io.IOException {
      Delete result = new Delete(customerId, subscriptionId, deletionType);
      initialize(result);
      return result;
    }

    public class Delete extends ResellerRequest<Void> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}";

      /**
       * Cancels/Downgrades a subscription.
       *
       * Create a request for the method "subscriptions.delete".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       * <p> {@link Delete#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @param deletionType Whether the subscription is to be fully cancelled or downgraded
       * @since 1.13
       */
      protected Delete(String customerId, String subscriptionId, String deletionType) {
        super(Reseller.this, "DELETE", REST_PATH, null, Void.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
        this.deletionType = Preconditions.checkNotNull(deletionType, "Required parameter deletionType must be specified.");
      }

      @Override
      public Delete setAlt(String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Delete setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public Delete setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

      /** Whether the subscription is to be fully cancelled or downgraded */
      @com.google.api.client.util.Key
      private String deletionType;

      /** Whether the subscription is to be fully cancelled or downgraded
       */
      public String getDeletionType() {
        return deletionType;
      }

      /** Whether the subscription is to be fully cancelled or downgraded */
      public Delete setDeletionType(String deletionType) {
        this.deletionType = deletionType;
        return this;
      }

    }
    /**
     * Gets a subscription of the customer.
     *
     * Create a request for the method "subscriptions.get".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @return the request
     */
    public Get get(String customerId, String subscriptionId) throws java.io.IOException {
      Get result = new Get(customerId, subscriptionId);
      initialize(result);
      return result;
    }

    public class Get extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}";

      /**
       * Gets a subscription of the customer.
       *
       * Create a request for the method "subscriptions.get".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @since 1.13
       */
      protected Get(String customerId, String subscriptionId) {
        super(Reseller.this, "GET", REST_PATH, null, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Get setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public Get setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

    }
    /**
     * Creates/Transfers a subscription for the customer.
     *
     * Create a request for the method "subscriptions.insert".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param content the {@link com.google.api.services.reseller.model.Subscription}
     * @return the request
     */
    public Insert insert(String customerId, com.google.api.services.reseller.model.Subscription content) throws java.io.IOException {
      Insert result = new Insert(customerId, content);
      initialize(result);
      return result;
    }

    public class Insert extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions";

      /**
       * Creates/Transfers a subscription for the customer.
       *
       * Create a request for the method "subscriptions.insert".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       * <p> {@link Insert#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param content the {@link com.google.api.services.reseller.model.Subscription}
       * @since 1.13
       */
      protected Insert(String customerId, com.google.api.services.reseller.model.Subscription content) {
        super(Reseller.this, "POST", REST_PATH, content, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public Insert setAlt(String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public Insert setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /**
     * An auth token needed for transferring a subscription. Can be generated at
     * https://www.google.com/a/cpanel/customer-domain/TransferToken. Optional.
     */
      @com.google.api.client.util.Key
      private String customerAuthToken;

      /** An auth token needed for transferring a subscription. Can be generated at
     https://www.google.com/a/cpanel/customer-domain/TransferToken. Optional.
       */
      public String getCustomerAuthToken() {
        return customerAuthToken;
      }

      /**
     * An auth token needed for transferring a subscription. Can be generated at
     * https://www.google.com/a/cpanel/customer-domain/TransferToken. Optional.
     */
      public Insert setCustomerAuthToken(String customerAuthToken) {
        this.customerAuthToken = customerAuthToken;
        return this;
      }

    }
    /**
     * Lists subscriptions of a reseller, optionally filtered by a customer name prefix.
     *
     * Create a request for the method "subscriptions.list".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends ResellerRequest<com.google.api.services.reseller.model.Subscriptions> {

      private static final String REST_PATH = "subscriptions";

      /**
       * Lists subscriptions of a reseller, optionally filtered by a customer name prefix.
       *
       * Create a request for the method "subscriptions.list".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Reseller.this, "GET", REST_PATH, null, com.google.api.services.reseller.model.Subscriptions.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * An auth token needed if the customer is not a resold customer of this reseller. Can be
     * generated at https://www.google.com/a/cpanel/customer-domain/TransferToken.Optional.
     */
      @com.google.api.client.util.Key
      private String customerAuthToken;

      /** An auth token needed if the customer is not a resold customer of this reseller. Can be generated at
     https://www.google.com/a/cpanel/customer-domain/TransferToken.Optional.
       */
      public String getCustomerAuthToken() {
        return customerAuthToken;
      }

      /**
     * An auth token needed if the customer is not a resold customer of this reseller. Can be
     * generated at https://www.google.com/a/cpanel/customer-domain/TransferToken.Optional.
     */
      public List setCustomerAuthToken(String customerAuthToken) {
        this.customerAuthToken = customerAuthToken;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page in the list
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public List setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Maximum number of results to return */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of results to return

     [minimum: 1] [maximum: 100]
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /**
     * Prefix of the customer's domain name by which the subscriptions should be filtered. Optional
     */
      @com.google.api.client.util.Key
      private String customerNamePrefix;

      /** Prefix of the customer's domain name by which the subscriptions should be filtered. Optional
       */
      public String getCustomerNamePrefix() {
        return customerNamePrefix;
      }

      /**
     * Prefix of the customer's domain name by which the subscriptions should be filtered. Optional
     */
      public List setCustomerNamePrefix(String customerNamePrefix) {
        this.customerNamePrefix = customerNamePrefix;
        return this;
      }

    }
    /**
     * Starts paid service of a trial subscription
     *
     * Create a request for the method "subscriptions.startPaidService".
     *
     * This request holds the parameters needed by the the reseller server.  After setting any optional
     * parameters, call the {@link StartPaidService#execute()} method to invoke the remote operation.
     *
     * @param customerId Id of the Customer
     * @param subscriptionId Id of the subscription, which is unique for a customer
     * @return the request
     */
    public StartPaidService startPaidService(String customerId, String subscriptionId) throws java.io.IOException {
      StartPaidService result = new StartPaidService(customerId, subscriptionId);
      initialize(result);
      return result;
    }

    public class StartPaidService extends ResellerRequest<com.google.api.services.reseller.model.Subscription> {

      private static final String REST_PATH = "customers/{customerId}/subscriptions/{subscriptionId}/startPaidService";

      /**
       * Starts paid service of a trial subscription
       *
       * Create a request for the method "subscriptions.startPaidService".
       *
       * This request holds the parameters needed by the the reseller server.  After setting any
       * optional parameters, call the {@link StartPaidService#execute()} method to invoke the remote
       * operation. <p> {@link StartPaidService#initialize(AbstractGoogleClientRequest)} must be called
       * to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Id of the Customer
       * @param subscriptionId Id of the subscription, which is unique for a customer
       * @since 1.13
       */
      protected StartPaidService(String customerId, String subscriptionId) {
        super(Reseller.this, "POST", REST_PATH, null, com.google.api.services.reseller.model.Subscription.class);
        this.customerId = Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.subscriptionId = Preconditions.checkNotNull(subscriptionId, "Required parameter subscriptionId must be specified.");
      }

      @Override
      public StartPaidService setAlt(String alt) {
        return (StartPaidService) super.setAlt(alt);
      }

      @Override
      public StartPaidService setFields(String fields) {
        return (StartPaidService) super.setFields(fields);
      }

      @Override
      public StartPaidService setKey(String key) {
        return (StartPaidService) super.setKey(key);
      }

      @Override
      public StartPaidService setOauthToken(String oauthToken) {
        return (StartPaidService) super.setOauthToken(oauthToken);
      }

      @Override
      public StartPaidService setPrettyPrint(Boolean prettyPrint) {
        return (StartPaidService) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public StartPaidService setQuotaUser(String quotaUser) {
        return (StartPaidService) super.setQuotaUser(quotaUser);
      }

      @Override
      public StartPaidService setUserIp(String userIp) {
        return (StartPaidService) super.setUserIp(userIp);
      }

      /** Id of the Customer */
      @com.google.api.client.util.Key
      private String customerId;

      /** Id of the Customer
       */
      public String getCustomerId() {
        return customerId;
      }

      /** Id of the Customer */
      public StartPaidService setCustomerId(String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Id of the subscription, which is unique for a customer */
      @com.google.api.client.util.Key
      private String subscriptionId;

      /** Id of the subscription, which is unique for a customer
       */
      public String getSubscriptionId() {
        return subscriptionId;
      }

      /** Id of the subscription, which is unique for a customer */
      public StartPaidService setSubscriptionId(String subscriptionId) {
        this.subscriptionId = subscriptionId;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Reseller}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Reseller}. */
    @Override
    public Reseller build() {
      return new Reseller(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link ResellerRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setResellerRequestInitializer(
        ResellerRequestInitializer resellerRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(resellerRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
